'use strict';

var stream = require('stream');
var qs = require('qs');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

var qs__default = /*#__PURE__*/_interopDefaultCompat(qs);

const DEFAULT_BASE_URL = "https://api.stackexchange.com/2.3";
const DEFAULT_MAX_PAGE = 100;
class StackOverflowQuestionsCollatorFactory {
  requestParams;
  baseUrl;
  apiKey;
  apiAccessToken;
  teamName;
  maxPage;
  logger;
  type = "stack-overflow";
  constructor(options) {
    this.baseUrl = options.baseUrl;
    this.apiKey = options.apiKey;
    this.apiAccessToken = options.apiAccessToken;
    this.teamName = options.teamName;
    this.maxPage = options.maxPage;
    this.logger = options.logger.child({ documentType: this.type });
    this.requestParams = {
      order: "desc",
      sort: "activity",
      ...options.requestParams ?? {}
    };
    if (!options.requestParams?.site && this.baseUrl === DEFAULT_BASE_URL) {
      this.requestParams.site = "stackoverflow";
    }
  }
  static fromConfig(config, options) {
    const apiKey = config.getOptionalString("stackoverflow.apiKey");
    const apiAccessToken = config.getOptionalString(
      "stackoverflow.apiAccessToken"
    );
    const teamName = config.getOptionalString("stackoverflow.teamName");
    const baseUrl = config.getOptionalString("stackoverflow.baseUrl") || DEFAULT_BASE_URL;
    const maxPage = options.maxPage || DEFAULT_MAX_PAGE;
    const requestParams = config.getOptionalConfig("stackoverflow.requestParams")?.get();
    return new StackOverflowQuestionsCollatorFactory({
      baseUrl,
      maxPage,
      apiKey,
      apiAccessToken,
      teamName,
      requestParams,
      ...options
    });
  }
  async getCollator() {
    return stream.Readable.from(this.execute());
  }
  async *execute() {
    if (!this.baseUrl) {
      this.logger.debug(
        `No stackoverflow.baseUrl configured in your app-config.yaml`
      );
    }
    if (this.apiKey && this.teamName) {
      this.logger.debug(
        "Both stackoverflow.apiKey and stackoverflow.teamName configured in your app-config.yaml, apiKey must be removed before teamName will be used"
      );
    }
    try {
      if (Object.keys(this.requestParams).indexOf("key") >= 0) {
        this.logger.warn(
          "The API Key should be passed as a separate param to bypass encoding"
        );
        delete this.requestParams.key;
      }
    } catch (e) {
      this.logger.error(`Caught ${e}`);
    }
    const params = qs__default.default.stringify(this.requestParams, {
      arrayFormat: "comma",
      addQueryPrefix: true
    });
    const apiKeyParam = this.apiKey ? `${params ? "&" : "?"}key=${this.apiKey}` : "";
    const teamParam = this.teamName ? `${params ? "&" : "?"}team=${this.teamName}` : "";
    const requestUrl = this.apiKey ? `${this.baseUrl}/questions${params}${apiKeyParam}` : `${this.baseUrl}/questions${params}${teamParam}`;
    let hasMorePages = true;
    let page = 1;
    while (hasMorePages) {
      if (page === this.maxPage) {
        this.logger.warn(
          `Over ${this.maxPage} requests to the Stack Overflow API have been made, which may not have been intended. Either specify requestParams that limit the questions returned, or configure a higher maxPage if necessary.`
        );
        break;
      }
      const res = await fetch(
        `${requestUrl}&page=${page}`,
        this.apiAccessToken ? {
          headers: {
            "X-API-Access-Token": this.apiAccessToken
          }
        } : void 0
      );
      const data = await res.json();
      for (const question of data.items ?? []) {
        yield {
          title: question.title,
          location: question.link,
          text: question.owner.display_name,
          tags: question.tags,
          answers: question.answer_count
        };
      }
      hasMorePages = data.has_more;
      page = page + 1;
    }
  }
}

exports.StackOverflowQuestionsCollatorFactory = StackOverflowQuestionsCollatorFactory;
//# sourceMappingURL=StackOverflowQuestionsCollatorFactory.cjs.js.map
